@@ -19,12 +19,13 @@
 
 """Provides the interface to the AT-SPI Registry."""
 
-__id__        = "$Id: atspi.py,v 1.38 2006/07/18 14:15:43 wwalker Exp $"
-__version__   = "$Revision: 1.38 $"
-__date__      = "$Date: 2006/07/18 14:15:43 $"
+__id__        = "$Id: atspi.py,v 1.45 2006/08/02 21:34:30 wwalker Exp $"
+__version__   = "$Revision: 1.45 $"
+__date__      = "$Date: 2006/08/02 21:34:30 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import signal
 import time
 
 import gobject
@@ -143,14 +144,10 @@ class Registry:
 
     def __blockPreventor(self):
         """[[[TODO: HACK to attempt to prevent deadlocks.  We call time.sleep
-        here as a means to sidestep the global interpreter lock (GIL).  Take
-        a look at the following URLs for more information:
-
-        http://mail.python.org/pipermail/python-list/2002-October/126632.html
-        http://twistedmatrix.com/pipermail/twisted-python/2005-July/011052.html
-        http://www.pyzine.com/Issue001/Section_Articles/article_ThreadingGlobalInterpreter.html"""
-
-        time.sleep(0.0001) # Attempt to sidestep GIL
+        here as a means to sidestep the global interpreter lock (GIL).]]]
+        """
+        if settings.gilSleepTime:
+            time.sleep(settings.gilSleepTime)
         return True
 
     def start(self):
@@ -164,17 +161,21 @@ class Registry:
         #
         if settings.useBonoboMain:
             debug.println(debug.LEVEL_CONFIGURATION,
-                          "atspi.start: using bonobo.main")
-            gobject.idle_add(self.__blockPreventor)
+                          "atspi.start: using bonobo.main; "
+                          + "gilSleepTime=%f" % settings.gilSleepTime)
+            if settings.gilSleepTime:
+                gobject.idle_add(self.__blockPreventor)
             bonobo.main()
         else:
             debug.println(debug.LEVEL_CONFIGURATION,
-                          "atspi.start: using our custom main loop")
+                          "atspi.start: using our custom main loop; "
+                          + "gilSleepTime=%f" % settings.gilSleepTime)
             self.running = True
             context = gobject.MainLoop().get_context()
             while self.running:
-                time.sleep(0.0001) # Attempt to sidestep GIL
-                dispatched = context.iteration(False)
+                if settings.gilSleepTime:
+                    time.sleep(settings.gilSleepTime)
+                context.iteration(False)
 
     def stop(self):
         """Unregisters any event or keystroke listeners registered with
@@ -277,11 +278,18 @@ class EventListener(Accessibility__POA.EventListener):
         self.__registered = False
 
     def notifyEvent(self, event):
+        if settings.timeoutCallback and (settings.timeoutTime > 0):
+            signal.signal(signal.SIGALRM, settings.timeoutCallback)
+            signal.alarm(settings.timeoutTime)
+
         try:
             self.callback(event)
         except:
             debug.printException(debug.LEVEL_WARNING)
 
+        if settings.timeoutCallback and (settings.timeoutTime > 0):
+            signal.alarm(0)
+
     def __del__(self):
         self.deregister()
 
@@ -356,11 +364,20 @@ class KeystrokeListener(Accessibility__POA.DeviceEventListener):
 
         Returns True if the event has been consumed.
         """
+        if settings.timeoutCallback and (settings.timeoutTime > 0):
+            signal.signal(signal.SIGALRM, settings.timeoutCallback)
+            signal.alarm(settings.timeoutTime)
+
         try:
-            return self.callback(event)
+            consumed = self.callback(event)
         except:
             debug.printException(debug.LEVEL_WARNING)
-            return False
+            consumed = False
+
+        if settings.timeoutCallback and (settings.timeoutTime > 0):
+            signal.alarm(0)
+
+        return consumed
 
     def __del__(self):
         self.deregister()
@@ -917,7 +934,10 @@ class Accessible:
 
         stateSet = self.accessible.getState()
         if stateSet:
-            state = stateSet._narrow(Accessibility.StateSet).getStates()
+            try:
+                state = stateSet._narrow(Accessibility.StateSet).getStates()
+            except:
+                state = []
         else:
             state = []
 
@@ -938,7 +958,10 @@ class Accessible:
         relationSet = self.accessible.getRelationSet()
 
         for relation in relationSet:
-            relations.append(relation._narrow(Accessibility.Relation))
+            try:
+                relations.append(relation._narrow(Accessibility.Relation))
+            except:
+                pass
 
         if settings.cacheValues:
             self.relations = relations
@@ -1035,7 +1058,10 @@ class Accessible:
         action = self.accessible.queryInterface("IDL:Accessibility/Action:1.0")
 
         if action:
-            action = action._narrow(Accessibility.Action)
+            try:
+                action = action._narrow(Accessibility.Action)
+            except:
+                action = None
 
         if action and settings.cacheValues:
             self.action = action
@@ -1052,7 +1078,10 @@ class Accessible:
             "IDL:Accessibility/Component:1.0")
 
         if component:
-            component = component._narrow(Accessibility.Component)
+            try:
+                component = component._narrow(Accessibility.Component)
+            except:
+                component = None
 
         if component and settings.cacheValues:
             self.component = component
@@ -1069,7 +1098,10 @@ class Accessible:
             "IDL:Accessibility/Hypertext:1.0")
 
         if hypertext:
-            hypertext = hypertext._narrow(Accessibility.Hypertext)
+            try:
+                hypertext = hypertext._narrow(Accessibility.Hypertext)
+            except:
+                hypertext = None
 
         if hypertext and settings.cacheValues:
             self.hypertext = hypertext
@@ -1086,7 +1118,10 @@ class Accessible:
             "IDL:Accessibility/Image:1.0")
 
         if image:
-            image = image._narrow(Accessibility.Image)
+            try:
+                image = image._narrow(Accessibility.Image)
+            except:
+                image = None
 
         if image and settings.cacheValues:
             self.image = image
@@ -1103,7 +1138,10 @@ class Accessible:
             "IDL:Accessibility/Selection:1.0")
 
         if selection:
-            selection = selection._narrow(Accessibility.Selection)
+            try:
+                selection = selection._narrow(Accessibility.Selection)
+            except:
+                selection = None
 
         if selection and settings.cacheValues:
             self.selection = selection
@@ -1119,7 +1157,10 @@ class Accessible:
         table = self.accessible.queryInterface("IDL:Accessibility/Table:1.0")
 
         if table:
-            table = table._narrow(Accessibility.Table)
+            try:
+                table = table._narrow(Accessibility.Table)
+            except:
+                table = None
 
         if table and settings.cacheValues:
             self.table = table
@@ -1135,7 +1176,10 @@ class Accessible:
         text = self.accessible.queryInterface("IDL:Accessibility/Text:1.0")
 
         if text:
-            text = text._narrow(Accessibility.Text)
+            try:
+                text = text._narrow(Accessibility.Text)
+            except:
+                text = None
 
         if text and settings.cacheValues:
             self.text = text
@@ -1151,7 +1195,10 @@ class Accessible:
         value = self.accessible.queryInterface("IDL:Accessibility/Value:1.0")
 
         if value:
-            value = value._narrow(Accessibility.Value)
+            try:
+                value = value._narrow(Accessibility.Value)
+            except:
+                value = None
 
         if value and settings.cacheValues:
             self.value = value
@@ -1220,8 +1267,8 @@ class Accessible:
         Arguments:
         - index: an integer specifying which child to obtain
 
-        Returns the child at the given index.  [[[TODO: WDW - what to do
-        when an index out of bounds occurs?  Logged as bugzilla bug 319738.]]]
+        Returns the child at the given index or raise an exception if the
+        index is out of bounds or the child is invalid.
         """
 
         # [[[TODO: WDW - the AT-SPI appears to give us a different accessible
@@ -1232,12 +1279,17 @@ class Accessible:
         # Save away details we now know about this child
         #
         newChild = None
-        accChild = self.accessible.getChildAtIndex(index)
-        if accChild:
-            newChild = Accessible.makeAccessible(accChild)
-            newChild.index = index
-            newChild.parent = self
-            newChild.app = self.app
+        if index >= 0 and index < self.accessible.childCount:
+            accChild = self.accessible.getChildAtIndex(index)
+            if accChild:
+                newChild = Accessible.makeAccessible(accChild)
+                newChild.index = index
+                newChild.parent = self
+                newChild.app = self.app
+
+        if not newChild:
+            raise Exception("Invalid child at index: %d" % index)
+
         return newChild
 
     def toString(self, indent="", includeApp=True):
@@ -1379,7 +1431,6 @@ def __test():
     registry.start()
 
 if __name__ == "__main__":
-    import signal
     signal.signal(signal.SIGINT, __shutdownAndExit)
     signal.signal(signal.SIGQUIT, __shutdownAndExit)
     __test()
