@@ -24,9 +24,9 @@
 
 """Displays a GUI for the user to set Orca preferences."""
 
-__id__        = "$Id: orca_gui_prefs.py,v 1.49 2006/07/28 17:33:17 wwalker Exp $"
-__version__   = "$Revision: 1.49 $"
-__date__      = "$Date: 2006/07/28 17:33:17 $"
+__id__        = "$Id: orca_gui_prefs.py,v 1.50 2006/08/08 15:04:40 richb Exp $"
+__version__   = "$Revision: 1.50 $"
+__date__      = "$Date: 2006/08/08 15:04:40 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -396,8 +396,8 @@ class orcaSetupGUI(GladeWrapper):
         - combobox: the GtkComboBox to search.
         - str: the string to search for.
 
-        Returns the index of the first entry in combobox with str, on
-        None if not found.
+        Returns the index of the first entry in combobox with str, or
+        0 if not found.
         """
 
         model = combobox.get_model()
@@ -408,7 +408,7 @@ class orcaSetupGUI(GladeWrapper):
                 return i
             iter = model.iter_next(iter)
 
-        return None
+        return 0
 
     def _setupServers(self):
         """Get the list of speech servers for the current speech factory.
